require("os");
dofile("Wowcompat.lua");
dofile("LibStub.lua");
dofile("../LibRingCache-1.0.lua");

local types = {
	["string"]   = "STR",
	["table"]    = "TBL",
	["function"] = "FNC",
	["number"]   = "NUM",	
}
local function print_table(t)
	local i=0;
	for k,v in pairs(t) do
		print(types[type(k)], k, types[type(v)], v);
		i = i+1;
	end
	return i;
end

local function collect()
	collectgarbage("collect");
end

local LRC = LibStub("LibRingCache-1.0");

do
	local c = LRC:create(10);
	local i=0;
	
	print("--------------- Tests --------------");
	print("Table data");
	-- test inserts
	for i=0, 9, 1 do
		c:insert("k"..i, {i});
	end
	assert(print_table(c.data) == 10, "Expected 10 items");
	
	print("Table data after garbage collect");
	collect();
	assert(print_table(c.data) == 10, "Expected 10 items");
	
	local toadd = "l1";
	print("Table data after inserting one single item ("..toadd..")");
	c:insert(toadd, {"added"});
	assert(print_table(c.data) == 11, "Expected 11 items");
	
	print("Table data after garbage collect");
	collect();
	assert(print_table(c.data) == 10, "Expected 10 items");
	assert(c:get("k0") == nil, "Expected k0 to have been collected");
	
	do
		print("Inserting another item (and do a GC), but I save k1 in a variable first");
		local v0 = c:get("k1");
		c:insert("l2", {"added2"});
		collect();
		assert(print_table(c.data) == 11, "Expected 11 items");
		assert(c:get("k1") ~= nil, "Expected k1 to still be here");
	end
	
	print("Variable should be released now");
	collect();
	assert(print_table(c.data) == 10, "Expected 10 items");
	assert(c:get("k1") == nil, "Expected k1 to be nil");
	
	

	print("------------ Benchmarks ------------");
	local iterations = 10000;
	print(iterations.." inserts");
	-- bench inserts
	local t = os.clock();
	for i=0, iterations, 1 do
		c:insert("k"..i, {i});
	end
	print(os.clock()-t, "seconds");
	
	print(iterations.." gets");
	-- bench gets
	local t = os.clock();
	for i=0, iterations, 1 do
		c:get("k"..i, {i});
	end
	print(os.clock()-t, "seconds");
end